@@ -20,7 +20,7 @@
  * Author: Alexander Larsson <alexl@redhat.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,6 +33,7 @@
 #include <stdlib.h>
 
 #include "gdummyfile.h"
+#include "gfile.h"
 
 #include "gioalias.h"
 
@@ -87,9 +88,8 @@ g_dummy_file_finalize (GObject *object)
     _g_decoded_uri_free (dummy->decoded_uri);
   
   g_free (dummy->text_uri);
-  
-  if (G_OBJECT_CLASS (g_dummy_file_parent_class)->finalize)
-    (*G_OBJECT_CLASS (g_dummy_file_parent_class)->finalize) (object);
+
+  G_OBJECT_CLASS (g_dummy_file_parent_class)->finalize (object);
 }
 
 static void
@@ -144,10 +144,6 @@ g_dummy_file_get_basename (GFile *file)
 static char *
 g_dummy_file_get_path (GFile *file)
 {
-  GDummyFile *dummy = G_DUMMY_FILE (file);
-
-  if (dummy->decoded_uri)
-    return g_strdup (dummy->decoded_uri->path);
   return NULL;
 }
 
@@ -172,7 +168,8 @@ g_dummy_file_get_parent (GFile *file)
   char *uri;
   GDecodedUri new_decoded_uri;
 
-  if (dummy->decoded_uri == NULL)
+  if (dummy->decoded_uri == NULL ||
+      g_strcmp0 (dummy->decoded_uri->path, "/") == 0)
     return NULL;
 
   dirname = g_path_get_dirname (dummy->decoded_uri->path);
@@ -261,8 +258,7 @@ match_prefix (const char *path,
 }
 
 static gboolean
-g_dummy_file_contains_file (GFile *parent,
-			    GFile *descendant)
+g_dummy_file_prefix_matches (GFile *parent, GFile *descendant)
 {
   GDummyFile *parent_dummy = G_DUMMY_FILE (parent);
   GDummyFile *descendant_dummy = G_DUMMY_FILE (descendant);
@@ -425,7 +421,7 @@ g_dummy_file_file_iface_init (GFileIface *iface)
   iface->get_uri = g_dummy_file_get_uri;
   iface->get_parse_name = g_dummy_file_get_parse_name;
   iface->get_parent = g_dummy_file_get_parent;
-  iface->contains_file = g_dummy_file_contains_file;
+  iface->prefix_matches = g_dummy_file_prefix_matches;
   iface->get_relative_path = g_dummy_file_get_relative_path;
   iface->resolve_relative_path = g_dummy_file_resolve_relative_path;
   iface->get_child_for_display_name = g_dummy_file_get_child_for_display_name;
