@@ -19,6 +19,12 @@
  * be used for encoding validation purpose.
  */
 
+/* for WC_NO_BEST_FIT_CHARS */
+#ifndef WINVER
+# define WINVER 0x0500
+#endif
+
+#define STRICT
 #include <windows.h>
 #include <errno.h>
 #include <string.h>
@@ -36,10 +42,12 @@
 
 #define MB_CHAR_MAX 16
 
-#define UNICODE_MODE_BOM_DONE 1
-#define UNICODE_MODE_SWAPPED 2
+#define UNICODE_MODE_BOM_DONE   1
+#define UNICODE_MODE_SWAPPED    2
 
-#define UNICODE_FLAG_USE_BOM_ENDIAN 1
+#define FLAG_USE_BOM_ENDIAN     1
+#define FLAG_TRANSLIT           2 /* //TRANSLIT */
+#define FLAG_IGNORE             4 /* //IGNORE (not implemented) */
 
 #define return_error(code)  \
     do {                    \
@@ -180,21 +188,27 @@ static struct {
     {1200, "CP1200"},
     {1200, "UTF16LE"},
     {1200, "UTF-16LE"},
+    {1200, "UCS2LE"},
     {1200, "UCS-2LE"},
 
     {1201, "CP1201"},
     {1201, "UTF16BE"},
     {1201, "UTF-16BE"},
+    {1201, "UCS2BE"},
     {1201, "UCS-2BE"},
     {1201, "unicodeFFFE"},
 
     {12000, "CP12000"},
     {12000, "UTF32LE"},
     {12000, "UTF-32LE"},
+    {12000, "UCS4LE"},
+    {12000, "UCS-4LE"},
 
     {12001, "CP12001"},
     {12001, "UTF32BE"},
     {12001, "UTF-32BE"},
+    {12001, "UCS4BE"},
+    {12001, "UCS-4BE"},
 
 #ifndef GLIB_COMPILATION
     /*
@@ -205,13 +219,18 @@ static struct {
     {1201, "UTF-16"},
     {12001, "UTF32"},
     {12001, "UTF-32"},
+    {12001, "UCS-4"},
+    {12001, "UCS4"},
 #else
     /* Default is little endian, because the platform is */
     {1200, "UTF16"},
     {1200, "UTF-16"},
+    {1200, "UCS2"},
     {1200, "UCS-2"},
     {12000, "UTF32"},
     {12000, "UTF-32"},
+    {12000, "UCS4"},
+    {12000, "UCS-4"},
 #endif
 
     /* copy from libiconv `iconv -l` */
@@ -873,6 +892,7 @@ make_csconv(const char *_name)
     CPINFOEX cpinfoex;
     csconv_t cv;
     int use_compat = TRUE;
+    int flag = 0;
     char name[128];
     char *p;
 
@@ -883,11 +903,15 @@ make_csconv(const char *_name)
     {
         if (_stricmp(p + 2, "nocompat") == 0)
             use_compat = FALSE;
+        else if (_stricmp(p + 2, "translit") == 0)
+            flag |= FLAG_TRANSLIT;
+        else if (_stricmp(p + 2, "ignore") == 0)
+            flag |= FLAG_IGNORE;
         *p = 0;
     }
 
     cv.mode = 0;
-    cv.flags = 0;
+    cv.flags = flag;
     cv.mblen = NULL;
     cv.flush = NULL;
     cv.compat = NULL;
@@ -898,15 +922,19 @@ make_csconv(const char *_name)
         cv.wctomb = utf16_wctomb;
         if (_stricmp(name, "UTF-16") == 0 ||
 	    _stricmp(name, "UTF16") == 0 ||
-	    _stricmp(name, "UCS-2") == 0)
-            cv.flags |= UNICODE_FLAG_USE_BOM_ENDIAN;
+	    _stricmp(name, "UCS-2") == 0 ||
+	    _stricmp(name, "UCS2") == 0)
+            cv.flags |= FLAG_USE_BOM_ENDIAN;
     }
     else if (cv.codepage == 12000 || cv.codepage == 12001)
     {
         cv.mbtowc = utf32_mbtowc;
         cv.wctomb = utf32_wctomb;
-        if (_stricmp(name, "UTF-32") == 0 || _stricmp(name, "UTF32") == 0)
-            cv.flags |= UNICODE_FLAG_USE_BOM_ENDIAN;
+        if (_stricmp(name, "UTF-32") == 0 ||
+	    _stricmp(name, "UTF32") == 0 ||
+	    _stricmp(name, "UCS-4") == 0 ||
+	    _stricmp(name, "UCS4") == 0)
+            cv.flags |= FLAG_USE_BOM_ENDIAN;
     }
     else if (cv.codepage == 65001)
     {
@@ -1061,7 +1089,7 @@ static void
 check_utf_bom(rec_iconv_t *cd, ushort *wbuf, int *wbufsize)
 {
     /* If we have a BOM, trust it, despite what the caller said */
-    if (wbuf[0] == 0xFFFE && (cd->from.flags & UNICODE_FLAG_USE_BOM_ENDIAN))
+    if (wbuf[0] == 0xFFFE && (cd->from.flags & FLAG_USE_BOM_ENDIAN))
     {
         /* swap endian: 1200 <-> 1201 or 12000 <-> 12001 */
         cd->from.codepage ^= 1;
@@ -1334,6 +1362,12 @@ kernel_mbtowc(csconv_t *cv, const uchar *buf, int bufsize, ushort *wbuf, int *wb
     len = cv->mblen(cv, buf, bufsize);
     if (len == -1)
         return -1;
+    /* If converting from ASCII, reject 8bit
+     * chars. MultiByteToWideChar() doesn't. Note that for ASCII we
+     * know that the mblen function is sbcs_mblen() so len is 1.
+     */
+    if (cv->codepage == 20127 && buf[0] >= 0x80)
+        return_error(EILSEQ);
     *wbufsize = MultiByteToWideChar(cv->codepage, mbtowc_flags (cv->codepage),
             (const char *)buf, len, (wchar_t *)wbuf, *wbufsize);
     if (*wbufsize == 0)
@@ -1345,13 +1379,22 @@ static int
 kernel_wctomb(csconv_t *cv, ushort *wbuf, int wbufsize, uchar *buf, int bufsize)
 {
     BOOL usedDefaultChar = 0;
+    BOOL *p = NULL;
+    int flags = 0;
     int len;
 
     if (bufsize == 0)
         return_error(E2BIG);
-    len = WideCharToMultiByte(cv->codepage, 0,
-            (const wchar_t *)wbuf, wbufsize, (char *)buf, bufsize, NULL,
-            must_use_null_useddefaultchar(cv->codepage) ? NULL : &usedDefaultChar);
+    if (!must_use_null_useddefaultchar(cv->codepage))
+    {
+        p = &usedDefaultChar;
+#ifdef WC_NO_BEST_FIT_CHARS
+        if (!(cv->flags & FLAG_TRANSLIT))
+            flags |= WC_NO_BEST_FIT_CHARS;
+#endif
+    }
+    len = WideCharToMultiByte(cv->codepage, flags,
+            (const wchar_t *)wbuf, wbufsize, (char *)buf, bufsize, NULL, p);
     if (len == 0)
     {
         if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
